"use strict";
// Copyright (c) Microsoft Corporation. All rights reserved. Licensed under the MIT license.
// See LICENSE in the project root for license information.
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeResolverCacheFromLockfileAsync = void 0;
const helpers_1 = require("./helpers");
function isPackageCompatible(pack, platformInfo) {
    var _a, _b, _c;
    if ((_a = pack.os) === null || _a === void 0 ? void 0 : _a.every((value) => value.toLowerCase() !== platformInfo.os)) {
        return false;
    }
    if ((_b = pack.cpu) === null || _b === void 0 ? void 0 : _b.every((value) => value.toLowerCase() !== platformInfo.cpu)) {
        return false;
    }
    if ((_c = pack.libc) === null || _c === void 0 ? void 0 : _c.every((value) => value.toLowerCase() !== platformInfo.libc)) {
        return false;
    }
    return true;
}
function extractBundledDependencies(contexts, context) {
    var _a;
    const { nestedPackageDirs } = context;
    if (!nestedPackageDirs) {
        return;
    }
    for (let i = nestedPackageDirs.length - 1; i >= 0; i--) {
        const nestedDir = nestedPackageDirs[i];
        if (!nestedDir.startsWith('node_modules/')) {
            continue;
        }
        const isScoped = nestedDir.charAt(/* 'node_modules/'.length */ 13) === '@';
        let index = nestedDir.indexOf('/', 13);
        if (isScoped) {
            index = nestedDir.indexOf('/', index + 1);
        }
        const name = index === -1 ? nestedDir.slice(13) : nestedDir.slice(13, index);
        if (name.startsWith('.')) {
            continue;
        }
        // Remove this nested package from the list
        nestedPackageDirs.splice(i, 1);
        const remainder = index === -1 ? '' : nestedDir.slice(index + 1);
        const nestedRoot = `${context.descriptionFileRoot}/node_modules/${name}`;
        let nestedContext = contexts.get(nestedRoot);
        if (!nestedContext) {
            nestedContext = {
                descriptionFileRoot: nestedRoot,
                descriptionFileHash: undefined,
                isProject: false,
                name,
                deps: new Map(),
                ordinal: -1
            };
            contexts.set(nestedRoot, nestedContext);
        }
        context.deps.set(name, nestedRoot);
        if (remainder) {
            (_a = nestedContext.nestedPackageDirs) !== null && _a !== void 0 ? _a : (nestedContext.nestedPackageDirs = []);
            nestedContext.nestedPackageDirs.push(remainder);
        }
    }
}
/**
 * Copied from `@rushstack/node-core-library/src/Path.ts` to avoid expensive dependency
 * @param path - Path using backslashes as path separators
 * @returns The same string using forward slashes as path separators
 */
function convertToSlashes(path) {
    return path.replace(/\\/g, '/');
}
/**
 * Given a lockfile and information about the workspace and platform, computes the resolver cache file.
 * @param params - The options for computing the resolver cache
 * @returns A promise that resolves with the resolver cache file
 */
async function computeResolverCacheFromLockfileAsync(params) {
    var _a;
    const { platformInfo, projectByImporterPath, lockfile, afterExternalPackagesAsync } = params;
    // Needs to be normalized to `/` for path.posix.join to work correctly
    const workspaceRoot = convertToSlashes(params.workspaceRoot);
    // Needs to be normalized to `/` for path.posix.join to work correctly
    const commonPrefixToTrim = convertToSlashes(params.commonPrefixToTrim);
    const contexts = new Map();
    const missingOptionalDependencies = new Set();
    // Enumerate external dependencies first, to simplify looping over them for store data
    for (const [key, pack] of lockfile.packages) {
        let name = pack.name;
        const descriptionFileRoot = (0, helpers_1.getDescriptionFileRootFromKey)(workspaceRoot, key, name);
        // Skip optional dependencies that are incompatible with the current environment
        if (pack.optional && !isPackageCompatible(pack, platformInfo)) {
            missingOptionalDependencies.add(descriptionFileRoot);
            continue;
        }
        const integrity = (_a = pack.resolution) === null || _a === void 0 ? void 0 : _a.integrity;
        if (!name && key.startsWith('/')) {
            const versionIndex = key.indexOf('@', 2);
            name = key.slice(1, versionIndex);
        }
        if (!name) {
            throw new Error(`Missing name for ${key}`);
        }
        const context = {
            descriptionFileRoot,
            descriptionFileHash: integrity,
            isProject: false,
            name,
            deps: new Map(),
            ordinal: -1,
            optional: pack.optional
        };
        contexts.set(descriptionFileRoot, context);
        if (pack.dependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, pack.dependencies, context);
        }
        if (pack.optionalDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, pack.optionalDependencies, context);
        }
    }
    if (afterExternalPackagesAsync) {
        await afterExternalPackagesAsync(contexts, missingOptionalDependencies);
    }
    for (const context of contexts.values()) {
        if (context.nestedPackageDirs) {
            extractBundledDependencies(contexts, context);
        }
    }
    // Add the data for workspace projects
    for (const [importerPath, importer] of lockfile.importers) {
        // Ignore the root project. This plugin assumes you don't have one.
        // A non-empty root project results in global dependency hoisting, and that's bad for strictness.
        if (importerPath === '.') {
            continue;
        }
        const project = projectByImporterPath.findChildPath(importerPath);
        if (!project) {
            throw new Error(`Missing project for importer ${importerPath}`);
        }
        const descriptionFileRoot = convertToSlashes(project.projectFolder);
        const context = {
            descriptionFileRoot,
            descriptionFileHash: undefined, // Not needed anymore
            name: project.packageJson.name,
            isProject: true,
            deps: new Map(),
            ordinal: -1
        };
        contexts.set(descriptionFileRoot, context);
        if (importer.dependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.dependencies, context);
        }
        if (importer.devDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.devDependencies, context);
        }
        if (importer.optionalDependencies) {
            (0, helpers_1.resolveDependencies)(workspaceRoot, importer.optionalDependencies, context);
        }
    }
    let ordinal = 0;
    for (const context of contexts.values()) {
        context.ordinal = ordinal++;
    }
    // Convert the intermediate representation to the final cache file
    const serializedContexts = Array.from(contexts, (0, helpers_1.createContextSerializer)(missingOptionalDependencies, contexts, commonPrefixToTrim));
    const cacheFile = {
        basePath: commonPrefixToTrim,
        contexts: serializedContexts
    };
    return cacheFile;
}
exports.computeResolverCacheFromLockfileAsync = computeResolverCacheFromLockfileAsync;
//# sourceMappingURL=computeResolverCacheFromLockfileAsync.js.map